/*
 * LeftRegularEquationSolver.cpp
 *
 *  Created on: 4. 3. 2014
 *	  Author: tomas
 */

#include "LeftRegularEquationSolver.h"
#include "regexp/unbounded/UnboundedRegExpElements.h"

using namespace alib;
using namespace alphabet;
using namespace regexp;
using namespace std;

namespace equations
{

LeftRegularEquationSolver::LeftRegularEquationSolver( void )
{

}

LeftRegularEquationSolver::~LeftRegularEquationSolver( void )
{
	for( const auto & kv : m_eqFinal )
		delete kv.second;
}

regexp::UnboundedRegExpElement* LeftRegularEquationSolver::eliminate( void )
{
	// RegExpOptimize opt; // TODO uncomment when implemented

	for( auto itA = m_symbolsByDepth.rbegin( ); itA != m_symbolsByDepth.rend( ); itA ++ )
	{
		const Symbol & a = * itA;

		/*
		 * Apply Arden's Lemma
		 * A = A0 + B1 + C2
		 * => A = 10*B + 20*C
		 */
		regexp::UnboundedRegExpIteration* loop = new regexp::UnboundedRegExpIteration( * m_eqTransition[ a ][ a ] );

		// for all transitions from A apply Arden's Lemma
		for( auto itB = std::next( itA ) ; itB != m_symbolsByDepth.rend( ); itB ++ )
		{
			const Symbol & b = * itB;
			regexp::UnboundedRegExpConcatenation* concat = new regexp::UnboundedRegExpConcatenation( );
			concat->appendElement( * m_eqTransition[ a ][ b ] );
			concat->appendElement( * loop );
			m_eqTransition[ a ][ b ] = /* opt.optimize( */ concat /* ) */;
			// delete concat;
		}
		regexp::UnboundedRegExpConcatenation* concat = new regexp::UnboundedRegExpConcatenation( );
		concat->appendElement( * m_eqFinal[ a ] );
		concat->appendElement( * loop );
		m_eqFinal[ a ] = /* opt.optimize( */ concat /* ) */;
		//delete concat;

		delete loop;

		/*
		 * eliminate A from rest of the equations using this pattern:
		 * B->C = B->C + concatenate( B->A, A->C )
		 */
		for( auto itB = std::next( itA ); itB != m_symbolsByDepth.rend( ); itB ++ )
		{
			const Symbol & b = * itB;

			for( auto itC = std::next( itA ); itC != m_symbolsByDepth.rend( ); itC ++ )
			{
				const Symbol & c = * itC;

				regexp::UnboundedRegExpAlternation* alt = new regexp::UnboundedRegExpAlternation( );
				regexp::UnboundedRegExpConcatenation* concat = new regexp::UnboundedRegExpConcatenation( );
				concat->appendElement( * m_eqTransition[ a ][ c ] );
				concat->appendElement( * m_eqTransition[ b ][ a ] );
				alt->appendElement( * m_eqTransition[ b ][ c ] );
				alt->appendElement( * concat );
				m_eqTransition[ b ][ c ] = /* opt.optimize( */ alt /* ) */;
				// delete alt;
			}

			regexp::UnboundedRegExpAlternation* alt = new regexp::UnboundedRegExpAlternation( );
			regexp::UnboundedRegExpConcatenation* concat = new regexp::UnboundedRegExpConcatenation( );
			concat->appendElement( * m_eqFinal[ a ] );
			concat->appendElement( * m_eqTransition[ b ][ a ] );
			alt->appendElement( * m_eqFinal[ b ] );
			alt->appendElement( * concat );
			m_eqFinal[ b ] = /* opt.optimize( */ alt /* ) */;
			// delete alt;
		}

		// no need for this now, release some memory
		for( const auto & kv : m_eqTransition[ a ] )
			delete kv.second;
	}

	regexp::UnboundedRegExpElement* result = m_eqFinal[ * m_symbolsByDepth.begin( ) ];
	m_eqFinal[ * m_symbolsByDepth.begin( ) ] = NULL;

	return result;
}

} /* namespace equations */
