/*
 * RegularEquationSolver.cpp
 *
 *  Created on: 4. 3. 2014
 *	  Author: tomas
 */

#include "RegularEquationSolver.h"

#include "regexp/unbounded/UnboundedRegExpElements.h"

using namespace alib;
using namespace alphabet;
using namespace regexp;

namespace equations
{

RegularEquationSolver::RegularEquationSolver( void )
{

}

RegularEquationSolver::~RegularEquationSolver( void )
{

}

regexp::UnboundedRegExp RegularEquationSolver::solve( const Symbol & solveFor )
{
	if( ! isInSet( solveFor, m_symbols ) )
		throw exception::AlibException( "Trying to solve for symbol not present in equation system" );

	/*
	 * Firstly, organize states by depth so we can output better looking
	 * expressions. We need to solve equation system for automaton initial state,
	 * so lets start with the deepest ones and walk towards initial one.
	 */
	symbolsByDepth( solveFor );

	UnboundedRegExpElement* result = eliminate( );

	regexp::UnboundedRegExp ret( *result );
	return ret;
}

void RegularEquationSolver::addSymbol( const alphabet::Symbol & symb )
{
	m_symbols.insert( symb );

	for( const auto & s1 : m_symbols )
	{
		m_eqTransition[ symb ][ s1 ] = new regexp::UnboundedRegExpEmpty( );
		if( s1 != symb )
			m_eqTransition[ s1 ][ symb ] = new regexp::UnboundedRegExpEmpty( );
	}

	m_eqFinal[ symb ] = new regexp::UnboundedRegExpEmpty( );
}

void RegularEquationSolver::addEquation( const Symbol & from, const Symbol & to, const UnboundedRegExpElement* eq )
{
	// RegExpOptimize opt; //TODO uncomment when implemented

	if( ! isInSet( from, m_symbols ) )
		throw exception::AlibException( "Symbol '" + (std::string) from + "' not in equation system." );

	if( ! isInSet( to, m_symbols ) )
		throw exception::AlibException( "Symbol '" + (std::string) to + "' not in equation system." );

	regexp::UnboundedRegExpAlternation* alt = new regexp::UnboundedRegExpAlternation( );
	alt->appendElement( * eq );
	alt->appendElement( * m_eqTransition[ from ][ to ] );
	m_eqTransition[ from ][ to ] = /* opt.optimize( */ alt /* ) */;
	delete eq;
//	delete alt;
}

void RegularEquationSolver::addEquation( const Symbol & from, const UnboundedRegExpElement* eq )
{
	// RegExpOptimize opt; //TODO uncommend when implemented

	if( ! isInSet( from, m_symbols ) )
		throw exception::AlibException( "Symbol '" + (std::string) from + "' not in equation system." );

	regexp::UnboundedRegExpAlternation* alt = new regexp::UnboundedRegExpAlternation( );
	alt->appendElement( * eq );
	alt->appendElement( * m_eqFinal[ from ] );
	m_eqFinal[ from ] = /* opt.optimize( */ alt /* ) */;
	delete eq;
//	delete alt;
}

void RegularEquationSolver::symbolsByDepth( const Symbol & solveFor )
{
	std::map<alphabet::Symbol, bool> visited;
	std::queue<alphabet::Symbol> queue;

	for( const auto & symbol : m_symbols )
		visited[ symbol ] = false;

	visited[ solveFor ] = true;
	queue.push( solveFor );


	while( ! queue.empty( ) )
	{
		Symbol s = queue.front( );
		queue.pop( );

		m_symbolsByDepth.push_back( s );

		auto map_s = m_eqTransition.find( s );
		for( auto it = map_s->second.begin( ); it != map_s->second.end( ); it ++ )
		{
			// neighbouring if regexp between s and kv.first is not \0
			if( dynamic_cast<regexp::UnboundedRegExpEmpty*>( it->second ) == NULL && visited [ it->first ] == false )
			{
				visited[ it->first ] = true;
				queue.push( it->first );
			}
		}
	}
}


} /* namespace equations */
