/*
 * RightRegularEquationSolver.cpp
 *
 *  Created on: 4. 3. 2014
 *	  Author: tomas
 */

#include "RightRegularEquationSolver.h"
#include "regexp/unbounded/UnboundedRegExpElements.h"

using namespace alib;
using namespace alphabet;
using namespace regexp;
using namespace std;

namespace equations
{

RightRegularEquationSolver::RightRegularEquationSolver( void )
{

}

RightRegularEquationSolver::~RightRegularEquationSolver( void )
{
	for( const auto & kv : m_eqFinal )
		delete kv.second;
}

regexp::UnboundedRegExpElement* RightRegularEquationSolver::eliminate( void )
{
	// RegExpOptimize opt; // TODO uncomment when implemented

	for( auto itA = m_symbolsByDepth.rbegin( ); itA != m_symbolsByDepth.rend( ); itA ++ )
	{
		const Symbol & a = * itA;

		/*
		 * Apply Arden's Lemma
		 * A = 0A + 1B + 2C
		 * => A = 0*1B + 0*2C
		 */
		regexp::UnboundedRegExpIteration loop( * m_eqTransition[ a ][ a ] );

		// for all transitions from A apply Arden's Lemma
		for( auto itB = std::next( itA ) ; itB != m_symbolsByDepth.rend( ); itB ++ )
		{
			const alphabet::Symbol & b = * itB;
			regexp::UnboundedRegExpConcatenation concat;
			concat.appendElement( loop );
			concat.appendElement( * m_eqTransition[ a ][ b ] );
			m_eqTransition[ a ][ b ] = /* opt.optimize( */ concat.clone() /* ) */;
		}
		regexp::UnboundedRegExpConcatenation concat;
		concat.appendElement( loop );
		concat.appendElement( * m_eqFinal[ a ] );
		m_eqFinal[ a ] = /* opt.optimize( */ concat.clone() /* ) */;

		/*
		 * eliminate A from rest of the equations using this pattern:
		 * B->C = B->C + concatenate( B->A, A->C )
		 */
		for( auto itB = std::next( itA ); itB != m_symbolsByDepth.rend( ); itB ++ )
		{
			const alphabet::Symbol & b = * itB;

			for( auto itC = std::next( itA ); itC != m_symbolsByDepth.rend( ); itC ++ )
			{
				const alphabet::Symbol & c = * itC;

				regexp::UnboundedRegExpAlternation alt;
				regexp::UnboundedRegExpConcatenation concat;
				concat.appendElement( * m_eqTransition[ b ][ a ] );
				concat.appendElement( * m_eqTransition[ a ][ c ] );
				alt.appendElement( * m_eqTransition[ b ][ c ] );
				alt.appendElement( concat );
				m_eqTransition[ b ][ c ] = /*opt.optimize( */ alt.clone() /* ) */;
			}

			regexp::UnboundedRegExpAlternation alt;
			regexp::UnboundedRegExpConcatenation concat;
			concat.appendElement( * m_eqTransition[ b ][ a ] );
			concat.appendElement( * m_eqFinal[ a ] );
			alt.appendElement( * m_eqFinal[ b ] );
			alt.appendElement( concat );
			m_eqFinal[ b ] = /* opt.optimize( */ alt.clone() /* ) */;
		}

		// no need for this now, release some memory
		for( const auto & kv : m_eqTransition[ a ] )
			delete kv.second;
	}

	regexp::UnboundedRegExpElement* result = m_eqFinal[ * m_symbolsByDepth.begin( ) ];
	m_eqFinal[ * m_symbolsByDepth.begin( ) ] = NULL;

	return result;
}

} /* namespace equations */
