/*
 * BrzozowskiAlgebraic.cpp
 *
 *  Created on: 11. 2. 2014
 *	  Author: tomas
 */

#include "BrzozowskiAlgebraic.h"
#include "alphabet/Symbol.h"

using namespace alib;
using namespace automaton;
using namespace regexp;

namespace fa2re
{

BrzozowskiAlgebraic::BrzozowskiAlgebraic( const automaton::NFA & automaton ) : m_automaton( automaton )
{

}

BrzozowskiAlgebraic::~BrzozowskiAlgebraic( void )
{

}

regexp::UnboundedRegExp BrzozowskiAlgebraic::convert( void )
{
	equations::RightRegularEquationSolver solver;

	// initialize equations
	for( const auto & q : m_automaton.getStates( ) )
		solver.addSymbol( alphabet::Symbol( alphabet::LabeledSymbol( q.getName( ) ) ) );

	for( const auto & q : m_automaton.getStates( ) )
	{
		if( isInSet( q, m_automaton.getFinalStates( ) ) )
			solver.addEquation( alphabet::Symbol( alphabet::LabeledSymbol ( q.getName( ) ) ), new regexp::UnboundedRegExpEpsilon( ) );
		else
			solver.addEquation( alphabet::Symbol( alphabet::LabeledSymbol ( q.getName( ) ) ), new regexp::UnboundedRegExpEmpty( ) );
	}

	for( const auto & p : m_automaton.getStates( ) )
	{
		for( const auto & q : m_automaton.getStates( ) )
		{
			regexp::UnboundedRegExpAlternation* alt = new regexp::UnboundedRegExpAlternation( );
			alt->appendElement( regexp::UnboundedRegExpEmpty { } );

			for( const auto & symbol : m_automaton.getInputAlphabet( ) ) {
				auto toStates = m_automaton.getTransitionsFromState(p)[std::make_pair(p, symbol)];
				if ( isInSet( q , toStates ) )
					alt->appendElement( regexp::UnboundedRegExpSymbol { symbol } );
			}
			solver.addEquation( alphabet::Symbol( alphabet::LabeledSymbol ( p.getName( ) ) ), alphabet::Symbol( alphabet::LabeledSymbol( q.getName( ) ) ), alt );
		}
	}

	regexp::UnboundedRegExpAlternation alternation;
	for(const auto& initialSymbol : m_automaton.getInitialStates() ) {
		// set symbol for which the solver will solve equation system
		alphabet::Symbol tmp( alphabet::LabeledSymbol ( initialSymbol.getName( ) ) );

		alternation.appendElement( solver.solve( tmp ).getRegExp() );
	}

	return regexp::UnboundedRegExp { alternation };
}

} /* namespace fa2re */
