/*
 * BrzozowskiAlgebraic.h
 *
 *  Created on: 11. 2. 2014
 *	  Author: tomas
 */

#ifndef BRZOZOWSKIALGEBRAIC_H_
#define BRZOZOWSKIALGEBRAIC_H_

#include <deque>
#include <map>
#include <queue>

#include <regexp/unbounded/UnboundedRegExpElements.h>
#include <automaton/FSM/NFA.h>

#include "common/macros.h"
#include "../equations/RightRegularEquationSolver.h"

namespace fa2re
{

// http://cs.stackexchange.com/questions/2016/how-to-convert-finite-automata-to-regular-expressions
// https://github.com/ferno/greenery/blob/bcc0a136335edbe94cd7725fc6e8cce0268d850c/fsm.py

/**
 * Converts FA to RE using Brzozowski's algebraic method using right regular equations.
 * Source : Melichar 2.122
 */
class BrzozowskiAlgebraic
{
public:
	/**
	 * @param fsm Source automaton
	 */
	BrzozowskiAlgebraic( const automaton::NFA & fsm );

	~BrzozowskiAlgebraic( void );

	/**
	 * Performs conversion.
	 * @return regular expression equivalent to input automaton.
	 */
	regexp::UnboundedRegExp convert( void );

private:
	/**
	 * Input automaton
	 */
	const automaton::NFA & m_automaton;
};

} /* namespace fa2re */

#endif /* BRZOZOWSKIALGEBRAIC_H_ */
