/*
 * Glushkov.cpp
 *
 *  Created on: 11. 1. 2014
 *	  Author: tomas
 */

#include "Glushkov.h"
#include "label/Label.h"
#include "label/StringLabel.h"
#include "label/CharacterLabel.h"
#include "label/IntegerLabel.h"
#include "label/LabelPairLabel.h"

using namespace alib;
using namespace alphabet;
using namespace automaton;
using namespace regexp;
using namespace std;

namespace re2fa {

Glushkov::Glushkov( const regexp::UnboundedRegExp & re ) : m_re( re ) {

}

automaton::NFA Glushkov::convert( void )
{
	automaton::NFA automaton;

	// step 1
	for( auto const& symbol : m_re.getAlphabet( ) )
		automaton.addInputSymbol( symbol );

	// steps 2, 3, 4
	m_first = GlushkovTraversal::first( m_re );
	m_last = GlushkovTraversal::last( m_re );
	for( auto const& x : GlushkovTraversal::getSymbols( m_re ) )
		for( auto const& f : GlushkovTraversal::follow( m_re, x ) ) {
			m_pairs.insert( GlushkovPair( x, f ) );
			//std::cout << x.getSymbolPtr() << "|" << x.getSymbol() << "|" << x.getId() << "  +  ";
			//std::cout << f.getSymbolPtr() << "|" << f.getSymbol() << "|" << f.getId() << std::endl;
		}
	// \e in q0 check is in step 7

	// step 5
	State q0( label::Label( label::LabelPairLabel( std::make_pair( label::Label( label::CharacterLabel( 'q' ) ), label::Label( label::IntegerLabel ( 0 ) ) ) ) ) );
	automaton.addState( q0 );
	automaton.addInitialState( q0 );
	for( auto const& symbol : GlushkovTraversal::getSymbols( m_re ) )
	{
		State q( label::Label( label::LabelPairLabel( std::make_pair( label::Label( label::StringLabel( symbol.getInputSymbol( ) ) ), label::Label( label::IntegerLabel ( symbol.getId( ) ) ) ) ) ) );

		m_stateMap.insert( std::pair<GlushkovSymbol, State>( symbol, q ) );
		automaton.addState( q );
	}

	// step 6
	for( auto const& symbol : m_first )
	{
		State const& q = m_stateMap.find( symbol )->second;

		automaton.addTransition( q0, symbol.getInputSymbol( ), q );
	}

	for( auto const& pair : m_pairs )
	{
		State const& p = m_stateMap.find( pair.getFirst( ) )->second;
		State const& q = m_stateMap.find( pair.getSecond( ) )->second;

		automaton.addTransition( p, pair.getSecond( ).getInputSymbol( ), q );
	}

	// step 7
	for( auto const& symbol : m_last )
	{
		State const & q = m_stateMap.find( symbol )->second;
		automaton.addFinalState( q );
	}

	if( m_re.containsEmptyString( ) )
		automaton.addFinalState( q0 );

	return automaton;
}

} /* namespace conversions */
