/*
 * RegExpDerivation.cpp
 *
 *  Created on: 19. 1. 2014
 *      Author: Tomas Pecka
 */

#include "RegExpDerivation.h"

RegExpDerivation::RegExpDerivation(){}
RegExpDerivation::~RegExpDerivation(){}

regexp::RegExp RegExpDerivation::derivation(const regexp::RegExp& regexp, const string::LinearString& string)
{
    std::pair<regexp::RegExp, string::LinearString> out(regexp, string);
    regexp.getData().Accept((void*) &out, *this);
    return out.first;
}

void RegExpDerivation::Visit(void* userData, const regexp::FormalRegExp& regexp)
{
    std::pair<regexp::RegExp, string::LinearString> &ret = *(std::pair<regexp::RegExp, string::LinearString>*) userData;

    const regexp::FormalRegExpElement* newRegExp = regexp.getRegExp().clone();

    for(const auto& symbol : ret.second.getContent())
    {
        std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> out(symbol, nullptr);
        newRegExp->Accept((void*) &out, *this);
        delete newRegExp;
        newRegExp = out.second;
    }

    ret.first = regexp::RegExp{regexp::FormalRegExp(std::move(*newRegExp))};
    delete newRegExp;
}

void RegExpDerivation::Visit(void* userData, const regexp::FormalRegExpAlternation& alternation)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);

    static_cast<const regexp::FormalRegExpElement&>(alternation.getLeftElement()).Accept(userData, *this);
    const regexp::FormalRegExpElement* leftDerivative = out.second;

    static_cast<const regexp::FormalRegExpElement&>(alternation.getRightElement()).Accept(userData, *this);
    const regexp::FormalRegExpElement* rightDerivative = out.second;

    out.second = new regexp::FormalRegExpAlternation(std::move(*leftDerivative), std::move(*rightDerivative));
    delete leftDerivative;
    delete rightDerivative;
}

void RegExpDerivation::Visit(void* userData, const regexp::FormalRegExpConcatenation& concatenation)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);

    static_cast<const regexp::FormalRegExpElement&>(concatenation.getLeftElement()).Accept(userData, *this);
    const regexp::FormalRegExpElement* leftDerivative = out.second;
    const regexp::FormalRegExpElement* rightElement = concatenation.getRightElement().clone();
    out.second = new regexp::FormalRegExpConcatenation(std::move(*leftDerivative), std::move(*rightElement));
    delete leftDerivative;
    delete rightElement;

    if(concatenation.getLeftElement().containsEmptyString())
    {
        const regexp::FormalRegExpElement *alternation = out.second, *rightDerivative = nullptr;
        static_cast<const regexp::FormalRegExpElement&>(concatenation.getRightElement()).Accept(userData, *this);
        rightDerivative = out.second;

        out.second = new regexp::FormalRegExpAlternation(std::move(*alternation), std::move(*rightDerivative));
        delete alternation;
        delete rightDerivative;
    }
}

void RegExpDerivation::Visit(void* userData, const regexp::FormalRegExpIteration& iteration)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);

    static_cast<const regexp::FormalRegExpElement&>(iteration.getElement()).Accept(userData, *this);
    const regexp::FormalRegExpElement* iterDerivative = out.second;
    const regexp::FormalRegExpElement* iter = iteration.clone();

    out.second = new regexp::FormalRegExpConcatenation(std::move(*iterDerivative), std::move(*iter));
    delete iterDerivative;
    delete iter;
}

void RegExpDerivation::Visit(void* userData, const regexp::FormalRegExpSymbol& symbol)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);

    if(out.first == symbol.getSymbol())
        out.second = new regexp::FormalRegExpEpsilon();
    else
        out.second = new regexp::FormalRegExpEmpty();
}


void RegExpDerivation::Visit(void* userData, const regexp::FormalRegExpEpsilon& epsilon)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);
    out.second = new regexp::FormalRegExpEmpty();
}

void RegExpDerivation::Visit(void* userData, const regexp::FormalRegExpEmpty& empty)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);
    out.second = new regexp::FormalRegExpEmpty();
}

void RegExpDerivation::Visit(void* userData, const regexp::UnboundedRegExp& regexp)
{
    std::pair<regexp::RegExp, string::LinearString> &ret = *(std::pair<regexp::RegExp, string::LinearString>*) userData;

    const regexp::UnboundedRegExpElement* newRegExp = regexp.getRegExp().clone();

    for(const auto& symbol : ret.second.getContent())
    {
        std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> out(symbol, nullptr);
        newRegExp->Accept((void*) &out, *this);
        delete newRegExp;
        newRegExp = out.second;
    }

    ret.first = regexp::RegExp{regexp::UnboundedRegExp(std::move(*newRegExp))};
    delete newRegExp;
}

void RegExpDerivation::Visit(void* userData, const regexp::UnboundedRegExpAlternation& alternation)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);
    regexp::UnboundedRegExpAlternation* ret = new regexp::UnboundedRegExpAlternation();

    for(const auto& child : alternation.getElements())
    {
        static_cast<const regexp::UnboundedRegExpElement&>(*child).Accept(userData, *this);
        ret->appendElement(std::move(*out.second));
        delete out.second;
    }

    out.second = ret;
}

void RegExpDerivation::Visit(void* userData, const regexp::UnboundedRegExpConcatenation& concatenation)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);
    regexp::UnboundedRegExpAlternation* ret = new regexp::UnboundedRegExpAlternation();

    for(auto child = concatenation.getElements().begin(); child != concatenation.getElements().end(); child ++)
    {
        regexp::UnboundedRegExpConcatenation* concat = new regexp::UnboundedRegExpConcatenation();
        static_cast<const regexp::UnboundedRegExpElement&>(**child).Accept(userData, *this);
        concat->appendElement(std::move(*out.second));
        delete out.second;

        auto succeedingElement = child;
        while(++succeedingElement != concatenation.getElements().end())
            concat->appendElement(std::move(*(*succeedingElement)->clone()));

        ret->appendElement(std::move(*concat));
        delete concat;

        if((*child)->containsEmptyString())
            continue; // this IF construct is intentional "to match algorithm"
        break;
    }

    out.second = ret;
}

void RegExpDerivation::Visit(void* userData, const regexp::UnboundedRegExpIteration& iteration)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);

    static_cast<const regexp::UnboundedRegExpElement&>(iteration.getElement()).Accept(userData, *this);
    const regexp::UnboundedRegExpElement* iterDerivative = out.second;
    const regexp::UnboundedRegExpElement* iter = iteration.clone();

    regexp::UnboundedRegExpConcatenation* ret = new regexp::UnboundedRegExpConcatenation();
    ret->appendElement(std::move(*iterDerivative));
    ret->appendElement(std::move(*iter));
    delete iterDerivative;
    delete iter;

    out.second = ret;
}

void RegExpDerivation::Visit(void* userData, const regexp::UnboundedRegExpSymbol& symbol)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);

    if(out.first == symbol.getSymbol())
        out.second = new regexp::UnboundedRegExpEpsilon();
    else
        out.second = new regexp::UnboundedRegExpEmpty();
}

void RegExpDerivation::Visit(void* userData, const regexp::UnboundedRegExpEpsilon& epsilon)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);
    out.second = new regexp::UnboundedRegExpEmpty();
}

void RegExpDerivation::Visit(void* userData, const regexp::UnboundedRegExpEmpty& empty)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);
    out.second = new regexp::UnboundedRegExpEmpty();
}
