/*
 * RegExpDerivation.h
 *
 *  Created on: 19. 1. 2014
 *      Author: Tomas Pecka
 */

#ifndef REGEXPDERIVATION_H_
#define REGEXPDERIVATION_H_

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExpElements.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>

#include <string/LinearString.h>
#include <string/StringBase.h>

/**
 * Calculates derivation of regular expression.
 *
 * Sources:
 *  - Melichar, definition 2.91 in chapter 2.4.3
 *  - Brzozowski, J. A. - Derivatives of regular expressions (1964)
 */
class RegExpDerivation : public regexp::VisitableRegExpBase::visitor_type, regexp::FormalRegExpElement::visitor_type, regexp::UnboundedRegExpElement::visitor_type
{
public:
    RegExpDerivation();
    ~RegExpDerivation();
    regexp::RegExp derivation(const regexp::RegExp& regexp, const string::LinearString& string);

private:
    void Visit(void*, const regexp::UnboundedRegExpAlternation& alternation);
    void Visit(void*, const regexp::UnboundedRegExpConcatenation& concatenation);
    void Visit(void*, const regexp::UnboundedRegExpIteration& iteration);
    void Visit(void*, const regexp::UnboundedRegExpSymbol& symbol);
    void Visit(void*, const regexp::UnboundedRegExpEpsilon& epsilon);
    void Visit(void*, const regexp::UnboundedRegExpEmpty& empty);

    void Visit(void*, const regexp::FormalRegExpAlternation& alternation);
    void Visit(void*, const regexp::FormalRegExpConcatenation& concatenation);
    void Visit(void*, const regexp::FormalRegExpIteration& iteration);
    void Visit(void*, const regexp::FormalRegExpSymbol& symbol);
    void Visit(void*, const regexp::FormalRegExpEpsilon& epsilon);
    void Visit(void*, const regexp::FormalRegExpEmpty& empty);

    void Visit(void*, const regexp::UnboundedRegExp& empty);
    void Visit(void*, const regexp::FormalRegExp& empty);
};

#endif /* REGEXPDERIVATION_H_ */
