/*
 * RegExpIntegral.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Tomas Pecka
 */

#include "RegExpIntegral.h"

RegExpIntegral::RegExpIntegral(){}
RegExpIntegral::~RegExpIntegral(){}

regexp::RegExp RegExpIntegral::integral(const regexp::RegExp& regexp, const string::LinearString& string)
{
    std::pair<regexp::RegExp, string::LinearString> out(regexp, string);
    regexp.getData().Accept((void*) &out, *this);
    return out.first;
}

void RegExpIntegral::Visit(void* userData, const regexp::FormalRegExp& regexp)
{
    std::pair<regexp::RegExp, string::LinearString> &ret = *(std::pair<regexp::RegExp, string::LinearString>*) userData;

    const regexp::FormalRegExpElement* newRegExp = regexp.getRegExp().clone();

    for(auto it = ret.second.getContent().begin(); it != ret.second.getContent().end(); it ++)
    {
        const auto& symbol = *it;

        std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> out(symbol, nullptr);
        newRegExp->Accept((void*) &out, *this);
        delete newRegExp;
        newRegExp = out.second;
    }

    ret.first = regexp::RegExp{regexp::FormalRegExp(std::move(*newRegExp))};
    delete newRegExp;
}

void RegExpIntegral::Visit(void* userData, const regexp::FormalRegExpAlternation& alternation)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);

    static_cast<const regexp::FormalRegExpElement&>(alternation.getLeftElement()).Accept(userData, *this);
    const regexp::FormalRegExpElement* leftIntegral = out.second;

    static_cast<const regexp::FormalRegExpElement&>(alternation.getRightElement()).Accept(userData, *this);
    const regexp::FormalRegExpElement* rightIntegral = out.second;

    out.second = new regexp::FormalRegExpAlternation(std::move(*leftIntegral), std::move(*rightIntegral));
    delete leftIntegral;
    delete rightIntegral;
}

void RegExpIntegral::Visit(void* userData, const regexp::FormalRegExpConcatenation& concatenation)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);

    regexp::FormalRegExpSymbol* symb = new regexp::FormalRegExpSymbol(out.first);
    regexp::FormalRegExpElement* concat = concatenation.clone();
    regexp::FormalRegExpConcatenation* ret = new regexp::FormalRegExpConcatenation(std::move(*symb), std::move(*concat));

    delete concat;
    delete symb;

    out.second = ret;
}

void RegExpIntegral::Visit(void* userData, const regexp::FormalRegExpIteration& iteration)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);

    regexp::FormalRegExpSymbol* symb = new regexp::FormalRegExpSymbol(out.first);
    regexp::FormalRegExpElement* iter = iteration.clone();
    regexp::FormalRegExpConcatenation* ret = new regexp::FormalRegExpConcatenation(std::move(*symb), std::move(*iter));

    delete iter;
    delete symb;

    out.second = ret;
}

void RegExpIntegral::Visit(void* userData, const regexp::FormalRegExpSymbol& symbol)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);

    regexp::FormalRegExpSymbol* symb = new regexp::FormalRegExpSymbol(out.first);
    regexp::FormalRegExpElement* symb2 = symbol.clone();
    regexp::FormalRegExpConcatenation* ret = new regexp::FormalRegExpConcatenation(std::move(*symb), std::move(*symb2));

    delete symb2;
    delete symb;

    out.second = ret;
}


void RegExpIntegral::Visit(void* userData, const regexp::FormalRegExpEpsilon& epsilon)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);
    regexp::FormalRegExpSymbol* symb = new regexp::FormalRegExpSymbol(out.first);
    out.second = symb;
}

void RegExpIntegral::Visit(void* userData, const regexp::FormalRegExpEmpty& empty)
{
    std::pair<alphabet::Symbol, regexp::FormalRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::FormalRegExpElement*>*) userData);
    out.second = new regexp::FormalRegExpEmpty();
}


void RegExpIntegral::Visit(void* userData, const regexp::UnboundedRegExp& regexp)
{
    std::pair<regexp::RegExp, string::LinearString> &ret = *(std::pair<regexp::RegExp, string::LinearString>*) userData;

    const regexp::UnboundedRegExpElement* newRegExp = regexp.getRegExp().clone();

    for(auto it = ret.second.getContent().begin(); it != ret.second.getContent().end(); it ++)
    {
        const auto& symbol = *it;

        std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> out(symbol, nullptr);
        newRegExp->Accept((void*) &out, *this);
        delete newRegExp;
        newRegExp = out.second;
    }

    ret.first = regexp::RegExp{regexp::UnboundedRegExp(std::move(*newRegExp))};
    delete newRegExp;
}

void RegExpIntegral::Visit(void* userData, const regexp::UnboundedRegExpAlternation& alternation)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);
    regexp::UnboundedRegExpAlternation* ret = new regexp::UnboundedRegExpAlternation();

    for(const auto& child : alternation.getElements())
    {
        static_cast<const regexp::UnboundedRegExpElement&>(*child).Accept(userData, *this);
        ret->appendElement(std::move(*out.second));
        delete out.second;
    }

    out.second = ret;
}

void RegExpIntegral::Visit(void* userData, const regexp::UnboundedRegExpConcatenation& concatenation)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);

    regexp::UnboundedRegExpSymbol* symb = new regexp::UnboundedRegExpSymbol(out.first);
    regexp::UnboundedRegExpElement* concat = concatenation.clone();
    regexp::UnboundedRegExpConcatenation* ret = new regexp::UnboundedRegExpConcatenation();
    ret->appendElement(std::move(*symb));
    ret->appendElement(std::move(*concat));

    delete concat;
    delete symb;

    out.second = ret;
}

void RegExpIntegral::Visit(void* userData, const regexp::UnboundedRegExpIteration& iteration)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);

    regexp::UnboundedRegExpSymbol* symb = new regexp::UnboundedRegExpSymbol(out.first);
    regexp::UnboundedRegExpElement* iter = iteration.clone();
    regexp::UnboundedRegExpConcatenation* ret = new regexp::UnboundedRegExpConcatenation();
    ret->appendElement(std::move(*symb));
    ret->appendElement(std::move(*iter));

    delete iter;
    delete symb;

    out.second = ret;
}

void RegExpIntegral::Visit(void* userData, const regexp::UnboundedRegExpSymbol& symbol)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);

    regexp::UnboundedRegExpSymbol* symb = new regexp::UnboundedRegExpSymbol(out.first);
    regexp::UnboundedRegExpElement* symb2 = symbol.clone();
    regexp::UnboundedRegExpConcatenation* ret = new regexp::UnboundedRegExpConcatenation();
    ret->appendElement(std::move(*symb));
    ret->appendElement(std::move(*symb2));

    delete symb2;
    delete symb;

    out.second = ret;
}

void RegExpIntegral::Visit(void* userData, const regexp::UnboundedRegExpEpsilon& epsilon)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);

    regexp::UnboundedRegExpSymbol* symb = new regexp::UnboundedRegExpSymbol(out.first);
    out.second = symb;
}

void RegExpIntegral::Visit(void* userData, const regexp::UnboundedRegExpEmpty& empty)
{
    std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*> &out = *((std::pair<alphabet::Symbol, regexp::UnboundedRegExpElement*>*) userData);
    out.second = new regexp::UnboundedRegExpEmpty();
}
